﻿using System;
using System.Drawing;
using System.IO;
using System.Security.Principal;
using System.Windows.Forms;
using Microsoft.Win32;

namespace LogonChanger
{
    public partial class Form1 : Form
    {
        private Image currPic;

        /// <summary>
        /// Constructor
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {
            Text = "W7C LogonUI Changer " + ProductVersion;
            // Populate size
            Size screenSize = SystemInformation.PrimaryMonitorSize;
            txtWidth.Text = screenSize.Width.ToString();
            txtHeight.Text = screenSize.Height.ToString();

            // Warn if not admin
            WindowsIdentity wi = WindowsIdentity.GetCurrent();
            var wp = new WindowsPrincipal(wi);
            if (!wp.IsInRole(WindowsBuiltInRole.Administrator))
            {
                showError("Please run this application as Admin!" + Environment.NewLine +
                          "You can do it by right-clicking and selecting \"Run as Administrator\"");
            }
        }

        /// <summary>
        /// Calculates best width & height to fit a ratio to a certain sized container
        /// </summary>
        /// <param name="containerSize"></param>
        /// <param name="targetRatio"></param>
        /// <returns></returns>
        private static Size bestFit(Size containerSize, double targetRatio)
        {
            var result = new Size(containerSize.Width, containerSize.Height);
            double containerRatio = (double) containerSize.Width/containerSize.Height;

            if (targetRatio > containerRatio)
            {
                result.Height = Convert.ToInt32(result.Width/targetRatio);
            }
            else if (containerRatio > targetRatio) // Container too wide
            {
                result.Width = Convert.ToInt32(targetRatio*result.Height);
            }
            return result;
        }

        /// <summary>
        /// Shows image on panel with the correct ratio. 
        /// </summary>
        /// <param name="f"></param>
        private void LoadImageToPanel(FileInfo f)
        {
            Image newPic = null;
            // Try loading the image. 
            // Will complain if the file can't be loaded as image.
            try
            {
                newPic = Image.FromFile(f.FullName);
            }
            catch
            {
                showError("Invalid image file!");
                return;
            }
            if (currPic != null)
                currPic.Dispose(); // dispose pic to release file handle
            currPic = newPic;
            Size resizeTo;
            try
            {
                resizeTo = bestFit(panel1.Size,
                                   Convert.ToDouble(txtWidth.Text)/Convert.ToDouble(txtHeight.Text));
            }
            catch
            {
                showError("Invalid resolution");
                return;
            }

            panel1.BackgroundImageLayout = ImageLayout.Center;
            panel1.BackgroundImage = ImageProcessor.resizeImage(currPic, resizeTo);
            lblInfo.Visible = false;
        }

        /// <summary>
        /// Apply button click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            // ---- Registry
            try
            {
                Registry.SetValue(@"HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\" +
                                  @"Authentication\LogonUI\Background", "OEMBackground", optEnable.Checked ? 1 : 0,
                                  RegistryValueKind.DWord);
                MessageBox.Show(
                    string.Format("Feature {0} in registry", optEnable.Checked ? "ENABLED" : "DISABLED"),
                    "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch
            {
                showError("Could not set registry key. Are you running the app as Administrator?");
                //return;
            }


            // ---- Pic Processing ----
            if (currPic == null || optDisable.Checked)
                return; // No pic selected or Disabling feature. Don't bother.

            Size screenSize;
            try
            {
                screenSize = new Size(Convert.ToInt32(txtWidth.Text), Convert.ToInt32(txtHeight.Text));
            }
            catch (Exception)
            {
                return;
            }

            using (JpgImage img = ImageProcessor.compressToJpeg(currPic, screenSize, 256000))
            {
                try
                {
                    img.writeStreamToFile(
                        Environment.ExpandEnvironmentVariables(
                            @"%windir%\system32\oobe\info\backgrounds\backgroundDefault.jpg"), true);
                }
                catch (Exception ex)
                {
                    showError("Error saving image:" + Environment.NewLine + ex.Message);
                    return;
                }
                MessageBox.Show(
                    string.Format("Image saved with JPEG quality {0} and size {1:0,000} bytes", img.Quality, img.Size),
                    "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        /// <summary>
        /// On closing, dispose pic to release file lock
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (currPic != null)
                currPic.Dispose();
        }

        /// <summary>
        /// Lazy shortcut
        /// </summary>
        /// <param name="text"></param>
        public static void showError(string text)
        {
            MessageBox.Show(text, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// Had to resort to this since drag & drop won't work with elevated privvy
        /// </summary>
        private void LoadImage()
        {
            openFileDialog1.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
            DialogResult res = openFileDialog1.ShowDialog();
            if (res == DialogResult.Cancel)
                return;

            var file = new FileInfo(openFileDialog1.FileName);
            if (file.Exists)
                LoadImageToPanel(file);
        }

        /// <summary>
        /// Debug use only!
        /// </summary>
        private void tester()
        {
            string s = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
            string[] files = Directory.GetFiles(s);
            var screenSize = new Size(Convert.ToInt32(txtWidth.Text), Convert.ToInt32(txtHeight.Text));
            foreach (string _file in files)
            {
                var fi = new FileInfo(_file);
                try
                {
                    LoadImageToPanel(fi);
                    using (JpgImage img = ImageProcessor.compressToJpeg(currPic, screenSize, 256000))
                    {
                        if (img.Size > 256000) // Uh oh...
                            Console.WriteLine("{0},{1:0,000},{2}", img.Quality, img.Size, fi.Name);
                    }
                }
                catch
                {
                } // who cares. move on...
            }
            Console.WriteLine("Done!");
        }

        private void panel1_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                var d = new DirectoryInfo(
                    Environment.ExpandEnvironmentVariables(@"%windir%\system32\oobe\info\backgrounds"));
                if (!d.Exists)
                    showError("Directory does not exist!");
                else
                {
                    System.Diagnostics.Process.Start(d.FullName);
                }
            }
            else
            {
                LoadImage();
            }
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("www.windows7center.com"); 
        }

        private void panel2_MouseClick(object sender, MouseEventArgs e)
        {
            System.Diagnostics.Process.Start("www.windows7center.com"); 
        }

        private void button2_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                    string.Format("This program was originally designed by GreenMartian and modified by djlegion."),
                    "About", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
}